require "tuning"

--MOD--
local function test_ground(inst, pt)
	local tiletype = GetGroundTypeAtPosition(pt)
	local ground_OK = tiletype ~= GROUND.ROCKY and tiletype ~= GROUND.ROAD and tiletype ~= GROUND.IMPASSABLE and
						tiletype ~= GROUND.UNDERROCK and tiletype ~= GROUND.WOODFLOOR and 
						tiletype ~= GROUND.CARPET and tiletype ~= GROUND.CHECKER and tiletype < GROUND.UNDERGROUND
	
	if ground_OK then
	    local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 4)
		local min_spacing = inst.components.deployable.min_spacing or 2

	    for k, v in pairs(ents) do
			if v ~= inst and v.entity:IsValid() and v.entity:IsVisible() and not v:HasTag("player") and not v.components.placer and v.parent == nil and not v:HasTag("FX") then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < min_spacing*min_spacing then
					return false
				end
			end
		end
		
		return true

	end
	return false
	
end
--/MOD--

local function MakeVegStats(seedweight, hunger, health, perish_time, sanity, cooked_hunger, cooked_health, cooked_perish_time, cooked_sanity)
	return {
		health = health,
		hunger = hunger,
		cooked_health = cooked_health,
		cooked_hunger = cooked_hunger,
		seed_weight = seedweight,
		perishtime = perish_time,
		cooked_perishtime = cooked_perish_time,
		sanity = sanity,
		cooked_sanity = cooked_sanity
		
	}
end

local COMMON = 3
local UNCOMMON = 1
local RARE = .5


VEGGIES = 
{
	
	cave_banana = MakeVegStats(0,	TUNING.CALORIES_SMALL,	TUNING.HEALING_TINY,	TUNING.PERISH_MED, 0,		
									TUNING.CALORIES_SMALL,	TUNING.HEALING_SMALL,	TUNING.PERISH_FAST, 0),

	carrot = MakeVegStats(COMMON,	TUNING.CALORIES_SMALL,	TUNING.HEALING_TINY,	TUNING.PERISH_MED, 0,		
									TUNING.CALORIES_SMALL,	TUNING.HEALING_SMALL,	TUNING.PERISH_FAST, 0),

	corn = MakeVegStats(COMMON, TUNING.CALORIES_MED,	TUNING.HEALING_SMALL,	TUNING.PERISH_MED, 0,		
								TUNING.CALORIES_SMALL,	TUNING.HEALING_SMALL,	TUNING.PERISH_SLOW, 0),
	
	pumpkin = MakeVegStats(UNCOMMON,	TUNING.CALORIES_LARGE,	TUNING.HEALING_SMALL,	TUNING.PERISH_MED, 0,		
										TUNING.CALORIES_LARGE,	TUNING.HEALING_MEDSMALL,	TUNING.PERISH_FAST, 0),
	
	eggplant = MakeVegStats(UNCOMMON,	TUNING.CALORIES_MED,	TUNING.HEALING_MEDSMALL,	TUNING.PERISH_MED, 0,		
										TUNING.CALORIES_MED,	TUNING.HEALING_MED,		TUNING.PERISH_FAST, 0),
	
	durian = MakeVegStats(RARE, TUNING.CALORIES_MED,	-TUNING.HEALING_SMALL,	TUNING.PERISH_MED, -TUNING.SANITY_TINY,
								TUNING.CALORIES_MED,	0,						TUNING.PERISH_FAST, -TUNING.SANITY_TINY),
	
	pomegranate = MakeVegStats(RARE,	TUNING.CALORIES_TINY,	TUNING.HEALING_SMALL,		TUNING.PERISH_FAST, 0,		
										TUNING.CALORIES_SMALL,	TUNING.HEALING_MED,	TUNING.PERISH_SUPERFAST, 0),
	
	dragonfruit = MakeVegStats(RARE,	TUNING.CALORIES_TINY,	TUNING.HEALING_SMALL,		TUNING.PERISH_FAST, 0,		
										TUNING.CALORIES_SMALL,	TUNING.HEALING_MED,	TUNING.PERISH_SUPERFAST, 0),

	berries = MakeVegStats(0,	TUNING.CALORIES_TINY,	0,	TUNING.PERISH_FAST, 0,
								TUNING.CALORIES_SMALL,	TUNING.HEALING_TINY,	TUNING.PERISH_SUPERFAST, 0),

}

local function MakeVeggie(name, has_seeds)

	local assets=
	{
		Asset("ANIM", "data/anim/"..name..".zip"),
		Asset("IMAGE", "data/inventoryimages/"..name..".tex"),
	}
	local assets_cooked=
	{
		Asset("ANIM", "data/anim/"..name..".zip"),
		Asset("IMAGE", "data/inventoryimages/"..name.."_cooked.tex"),
	}
	
	local assets_seeds =
	{
		Asset("ANIM", "data/anim/seeds.zip"),
		Asset("IMAGE", "data/inventoryimages/"..name.."_seeds.tex"),
		Asset("IMAGE", "data/inventoryimages/seeds_cooked.tex"),
	}

	local prefabs =
	{
		name.."_cooked",
		"spoiled_food",
	}    
	
	if has_seeds then
		table.insert(prefabs, name.."_seeds")
	end

	local function fn_seeds()
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
	    
		MakeInventoryPhysics(inst)
		inst.AnimState:SetBank("seeds")
		inst.AnimState:SetBuild("seeds")
		inst.AnimState:SetRayTestOnBB(true)
	    
		inst:AddComponent("edible")
		inst.components.edible.foodtype = "SEEDS"

		inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

		
		inst:AddComponent("tradable")
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
	    
		inst.AnimState:PlayAnimation("idle")
		inst.components.edible.healthvalue = TUNING.HEALING_TINY/2
		inst.components.edible.hungervalue = TUNING.CALORIES_TINY

		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(TUNING.PERISH_SUPERSLOW)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"
		
	    
		inst:AddComponent("cookable")
		inst.components.cookable.product = "seeds_cooked"
	    
		inst:AddComponent("bait")
		inst:AddComponent("plantable")
		inst.components.plantable.growtime = TUNING.SEEDS_GROW_TIME
		inst.components.plantable.product = name
		
		--MOD--
        if name == "durian" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("mandrake")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
            flower.SoundEmitter:PlaySound("dontstarve/creatures/mandrake/plant")
            flower.SoundEmitter:PlaySound("dontstarve/creatures/mandrake/plant_dirt")
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground
		
        elseif name == "carrot" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("rabbithole")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
					flower.entity:AddSoundEmitter()
					flower.SoundEmitter:PlaySound("dontstarve/creatures/mandrake/plant_dirt")
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground
		
        elseif name == "pomegranate" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("berrybush")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
					flower.components.pickable:OnTransplant()
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground
		elseif name == "cave_banana" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("cave_banana_tree")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
					flower.components.pickable:OnTransplant()
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground

        elseif name == "corn" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("mushtree_small")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
					flower.entity:AddSoundEmitter()
					flower.SoundEmitter:PlaySound("dontstarve/wilson/use_axe_mushroom")
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground

        elseif name == "dragonfruit" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("mushtree_medium")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
					flower.entity:AddSoundEmitter()
					flower.SoundEmitter:PlaySound("dontstarve/wilson/use_axe_mushroom")
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground

        elseif name == "eggplant" then
        local function CanDeploy(inst) return true end
            local function OnDeploy (inst, pt) 
                local flower = SpawnPrefab("mushtree_tall")
                if flower then
            		flower.Transform:SetPosition(pt.x, pt.y, pt.z)
                    inst.components.stackable:Get():Remove()
					flower.entity:AddSoundEmitter()
					flower.SoundEmitter:PlaySound("dontstarve/wilson/use_axe_mushroom")
                end
	

            end
	    inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = OnDeploy
		inst.components.deployable.test = test_ground
        end
       --/MOD--
	    
		return inst
	end

	local function fn(Sim)
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		MakeInventoryPhysics(inst)
		
		inst.AnimState:SetBank(name)
		inst.AnimState:SetBuild(name)
		inst.AnimState:PlayAnimation("idle")
	    
		inst:AddComponent("edible")
		inst.components.edible.healthvalue = VEGGIES[name].health
		inst.components.edible.hungervalue = VEGGIES[name].hunger
		inst.components.edible.sanityvalue = VEGGIES[name].sanity or 0		
		inst.components.edible.foodtype = "VEGGIE"

		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(VEGGIES[name].perishtime)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"
		
		inst:AddComponent("stackable")
		
		
		local is_big = name == "pumpkin" or name == "eggplant" or name == "durian"
		if not is_big then
			inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
		end

		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
	    
	    MakeSmallBurnable(inst)
		MakeSmallPropagator(inst)
		---------------------        

		inst:AddComponent("bait")
	    
		------------------------------------------------
		inst:AddComponent("tradable")
	    
		------------------------------------------------  
	    
		inst:AddComponent("cookable")
		inst.components.cookable.product = name.."_cooked"

		return inst
	end
	
	local function fn_cooked(Sim)
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		MakeInventoryPhysics(inst)
		
		inst.AnimState:SetBank(name)
		inst.AnimState:SetBuild(name)
		inst.AnimState:PlayAnimation("cooked")
	    
	    
		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(VEGGIES[name].cooked_perishtime)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"
	    
		inst:AddComponent("edible")
		inst.components.edible.healthvalue = VEGGIES[name].cooked_health
		inst.components.edible.hungervalue = VEGGIES[name].cooked_hunger
		inst.components.edible.sanityvalue = VEGGIES[name].cooked_sanity or 0
		inst.components.edible.foodtype = "VEGGIE"

		inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
		
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")

	    MakeSmallBurnable(inst)
		MakeSmallPropagator(inst)
		---------------------        

		inst:AddComponent("bait")
	    
		------------------------------------------------
		inst:AddComponent("tradable")

		return inst
	end
	local base = Prefab( "common/inventory/"..name, fn, assets, prefabs)
	
	local cooked = Prefab( "common/inventory/"..name.."_cooked", fn_cooked, assets_cooked)
	local seeds = has_seeds and Prefab( "common/inventory/"..name.."_seeds", fn_seeds, assets_seeds) or nil
	return base, cooked, seeds
		   
		   
		   
end


local prefs = {}
for veggiename,veggiedata in pairs(VEGGIES) do
	local veg, cooked, seeds = MakeVeggie(veggiename, veggiename ~= "berries")
	table.insert(prefs, veg)
	table.insert(prefs, cooked)
	if seeds then
		table.insert(prefs, seeds)
		--MOD--
		if veggiename == "durian" then
			table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "mandrake", "mandrake", "ground" ))
		elseif veggiename == "carrot" then
		    table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "rabbithole", "rabbit_hole", "anim" ))
		elseif veggiename == "pomegranate" then
		    table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "berrybush", "berrybush", "idle_dead" ))
		elseif veggiename == "cave_banana" then
		    table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "cave_banana_tree", "cave_banana_tree", "idle_loop" ))
                elseif veggiename == "corn" then   
                    table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "mushroom", "mushtree_small", "idle_planted" ))
                elseif veggiename == "dragonfruit" then
		    table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "mushroom", "mushroom_tree_med", "idle_planted" ))
                elseif veggiename == "eggplant" then
		    table.insert(prefs, MakePlacer( "common/"..veggiename.."_seeds_placer", "mushroom", "mushtree_tall", "idle_planted" ))
                end		
		--/MOD--
	end
end

return unpack(prefs) 
